// ##################################################################################### //
// ##### universal commands OTA, INITVAL, DATE/TIME, SETDST, ITEMS, LIST, DST, SUN ##### //
// ##################################################################################### //

#ifndef _MY_ST_STDCMD_H_
#define _MY_ST_STDCMD_H_

/** add your own "on command" code **/
uint8_t stdCommands() {  // called on received commands
  uint8_t done = 0;  // set if command is done
  /** OTA firmware up-load? **/
  if (strcmp(fldIn[1], "OTA") == 0) {
    // format from WebSocket Client:  "|to|WS|OTA|space|"
    sprintf(msg[0], "|%s|%s|%s|%s|5 min.time|to upload|", valIn[0], now_Address, fldIn[1], now_SSID);
    sendTo(msg[0]);
    delay(200);
    setOTA();
    done = 1;
  }
  else if (strcmp(fldIn[1], "ONLINE") == 0) {
    //  from WebSocket Client:  "|to|WS|ONLINE|<space>|", send isOnLine
    sprintf(msg[0], "|%s|%s%|%s|isOnLine|", valIn[0], now_Address, fldIn[1]);
    sendTo(msg[0]);
    done = 1;
  }      
  else if (strcmp(fldIn[1], "INITVAL") == 0) {
    // format from WebSocket Client:  "|to|WS|INITVAL|space|"
    readOrgValues();  // reset Item values, use the itemValOrg[][] values
    onStart();
    done = 2;
  }
  else if (strcmp(fldIn[1], "DATE") == 0 || strcmp(fldIn[1], "TIME") == 0) {
    //  from WebSocket Client:  "|to|WS|DATE or TIME|<space>|", send date and time
    sprintf(msg[0], "|%s|%s%|%s|LOC|dd-mm|%02d-%02d|hh:mm|%02d:%02d|", valIn[0], now_Address, fldIn[1], day(nowLOC), month(nowLOC), hour(nowLOC), minute(nowLOC));
    sendTo(msg[0]);
    done = 1;
  }      
  else if (strcmp(fldIn[1], "USEDST") == 0) {
    //  from WebSocket Client:  "|to|WS|USEDST|1|" 0: NO, 1: YES
    uint8_t d = atoi(valIn[1]);
    useDST = (d ? 1 : 0);
    saveIt = true;
    done = 2;
  }      
  else if (strcmp(fldIn[1], "ITEMS") == 0) {
    //  from WebSocket Client:  "|to|WS|ITEMS|<space>|"
    sprintf(msg[0], "|%s|%s%|%s|%d|1|%s|", valIn[0], now_Address, fldIn[1], nItems, itemValues[0][0]);
    for (uint8_t i = 1; i < nItems; i++) {
      sprintf(msg[1], "%d|%s|", (i + 1), itemValues[i][0]);
      strcat(msg[0], msg[1]);
    }
    sendTo(msg[0]);
    done = 1;
  }
  else if (strcmp(fldIn[1], "LIST") == 0) {
    //  from WebSocket Client:  "|to|WS|LIST|<item number>|"
    if (mergAll(atoi(valIn[1])-1)) {
      sprintf(msg[1],"|%s|%s|%s|%s%s", valIn[0], now_Address, fldIn[1], valIn[1], msg[0]);
      sendTo(msg[1]);
      done = 1;
    }
  }
  else if ((strcmp(fldIn[1], "DST") == 0) && useDST) {
    if (strcmp(fldIn[1], "DST") == 0) {
      // format: "|to|WS|DST|<space>|" then send: "|to|WS|isDTS|<val>|"
      sprintf(msg[0],"|%s|%s|%s| |isDST|%s|", valIn[0], now_Address, fldIn[1], (isDST ? "yes" : "no"));
      sendTo(msg[0]);
      done = 1;
    }
  }
  else if (strcmp(fldIn[1], "SUN") == 0) {
    // format: "|to|WS|SUN|<space>|" then send: "|to|from|sunrise|<val>|sunset|<val>|"
    sprintf(msg[0],"|%s|%s|%s| |rise|%02d:%02d|set|%02d:%02d|", valIn[0], now_Address, fldIn[1], hour(sun.rise), minute(sun.rise), hour(sun.set),minute(sun.set));
    sendTo(msg[0]);
    done = 1;
  }
  else if (strcmp(fldIn[1], "GETTIME") == 0) {
    // UTC/GMT timeStamp request from a Node: "|to|from|GETTIME|<space>|"
    sprintf(msg[0], "|%s|%s|GETTIME|UNIX-TS|UTC|%d|", valIn[0], now_Address, nowUTC);
    sendTo(msg[0]);
    done = 1;
  }
  else if (strcmp(fldIn[1], "SETTIME") == 0) {
    // UTC/GMT timeStamp received after a request: "|to|from|SETTIME|UTC-UNIXtimestamp|", set date and time, if no NTP 
    time_t t = atol(valIn[1]);
    if (t > 1514764800) {
      timeval n = {.tv_sec = t};
      settimeofday(&n, NULL);
    }
    done = 2;
  }      
  return done;
}

#endif
