////////////////////////////////////////////
// specify the funcion Items of this Node //
////////////////////////////////////////////

// ==========================================================//
//  the functions of this Node                               //
//  nigth lamp and (Not)AtHome                               //
// ==========================================================//
//  edit the specific function Items code in:                //
//  -----------------------------------------                //
//  "myST_Start.h"   is called from setup()                  //
//  "myST_Cmd.h"     is called on a received command         //
//  "myST_Actiont.h" is, once per minute, called from loop() //
//  if needed: extra onStart/Command/Action functions        //
//    bellow in this header file                             //
// ==========================================================//

#ifndef _MY_ST_NODE_H_
#define _MY_ST_NODE_H_

#include "myST_Gen.h"

const uint8_t OTA_Pin = 15,    // OTA upload: touch on PIN<n> or use the WS command "|OTA|" ...
              lampPin =  23,   // relays pin
              homePin = 4,    // manual (not)AtHome touch on PIN<n> command, flash r/g-LED and, if used, send command(s) to other Nodes
              greenPin =  22,  // home flash LED pin
              redPin =  21,    // Home flash LED pin
              flashOn = 50,    // Home flash ON millis
              threshold = 40;  // OTA and home touch threshold
const uint16_t flashOff = 1700;  // Home flash OFF millis
volatile time_t homeDetect = 0;  // Home touch counter
uint8_t flashPin = greenPin;     // default GREEN LED
boolean flashIsOn = false,       // used by code
        anodeOnVCC = true;       // false: pin-HIGH LED ON, true: pin-HIGH LED OFF  
time_t atHomeFlashTime,
       flashMillis = millis(),
       detectMillis = millis();
/** (Not)AtHome can send commnds to other Nodes or WebBrowser **/
// the send command as used on the Webbrowser, add always a set of two strings first:OFF, second:ON, ...
// ... if OFF(the first) the led is GREEN, if ON(the second) the led is RED
const uint8_t nToCmd = 1;  // the number of send commands in atHomCmd[][] to other Nodes atHomCmd[<nbr>][OFF] and atHomCmd[<nbr>][ON]
const char atHomeCmd[nToCmd][2][250] = {"|WS|000100|Hello!|AtHome|flash|GREEN|",       // as example a msg to browser (OFF=GREEN, AtHome)
                                        "|WS|000100|Good bye!|NotAtHome|flash|RED|"};  // (ON=RED, NotAtHome)
                                                     // messages as send from Webbrowser
                                                     // a OFF/ON command (as dummy example to test it on your webbrowser) 
                                                     //   {"|to|<ID>|<myub>|<priv>|name|<Item>|status|<OFF>|", "|to|<ID>|<myub>|<priv>|name|<Item>|status|<ON>|"} // by example: a door lock, heting, boiler, or ... ON/OFF
// a @ sign mean do NOT change this name
// =====================================
enum {  // this Node Items
  nLamp,   // Item 1
  nHome,   // Item 2
  nItems,  //@ the number of items in use on this Node
};
enum {  // basic field names
  nName,    //@
  nMode,    //@
  nStatus,  //@
  nPause    //@
};
enum {  // Item 1 special field names
  nRise1 = nPause + 1,
  nSet1,
  nFields1  //@
};
enum {  // Item 2 spesial field names
  nOnHH2 = nPause + 1,
  nOnMM2,
  nOffHH2,
  nOffMM2,
  nFields2  //@
};
const uint8_t nFields = (nFields1 > nFields2 ? nFields1 : nFields2),  // the maximum number of used Node Items fields
              itemFields[nItems] = {nFields1, nFields2},  // the number of fields Items[0], Item[1}, ... , Item[n]
              nMaxChars = (10 + 1);  // max number of chars in the <...Fields[]> and <...Values[]>
const uint16_t maxItemLen = nFields * nMaxChars + nFields + 1,
               eepromAddr[nItems] = {10, (10 + maxItemLen)};  // Item EEPROM start addresses (start after: OTA[0], address[1-6], and useDST[7])
char fldIn[nFields][nMaxChars],  // parse received message to fieldnames an fieldValues 
     valIn[nFields][nMaxChars];  //   ...
const char itemNames[nItems][nFields][nMaxChars] =  {{"name", "mode", "status", "pause", "rise",  "set"},
                                                     {"name", "mode", "status", "pause", "on-hh", "on-mm", "off-hh", "off-mm"}},
           itemValOrg[nItems][nFields][nMaxChars] = {{"lamp", "1",    "0",       "0",    "10",    "10"},
                                                     {"home", "1",    "0",       "0",    "23",    "00",    "07",     "00"}};
char itemValues[nItems][nFields][nMaxChars];
char cmdList[nItems][10] = {{"nothing|."}, {"nothing|."}};  // set additional command list or "nothing", put code in myST_Cmd.h->onCommand() 

/** general used functions **/
void fromSetup();  // wath to do from setup()?
void fromLoop_1();   // wath to do from loop() before onAction()?
void fromLoop_2();   // wath to do from loop() after onAction()?
uint8_t parseFldVal(char * mesage);  // used by onCommand() to parse the input char string

///////////////////////////////////////
/** spesial functions for this Thing **/
void atHomeInterrupt();
void sendAtHome();
void atHomeLedOff();
void atHomeFlash(); 
///////////////////////////////////////

/** to do from setup() here **/
void fromSetup() {
  /** init this Node Item(s) pins and ... here **/
  pinMode(lampPin, OUTPUT);
  pinMode(greenPin, OUTPUT);
  pinMode(redPin, OUTPUT);
  atHomeLedOff();
  touchAttachInterrupt(homePin, atHomeInterrupt, threshold);
}
 
/** to do from loop() here **/
void fromLoop_1() {
  // before onAction()
  if(homeDetect == 1) atHomeFlash();
  if (homeDetect && (getMillis(detectMillis) >= homeDetect)) homeDetect = 0;
}
 
/** to do from loop() here **/
void fromLoop_2() {
  if (getMillis(flashMillis) >= atHomeFlashTime) atHomeFlash();
}

/** parse the received message to fieldnames an fieldValues **/ 
uint8_t parseFldVal(char * mesage) {
  uint8_t n = 0;
  char * v;
  /** get the number of fields **/
  v = strchr(mesage, '|');
  while (v != NULL) {
    n++;
    v = strchr(v + 1, '|');
  }
  if (n > 3 && (n % 2 != 0)) { // <= 3:no data, 'n' not neg. wrong format
    n = (n - 1) / 2;;
    uint8_t i = 0;
    /** get the fields **/
    v = strtok (mesage, "|");
    while (v != NULL) {
      strcpy(fldIn[i], v);
      v = strtok (NULL, "|");
      strcpy(valIn[i++], v);
      v = strtok (NULL, "|");
    }
  }
  else n = 0;
  return n;
}

///////////////////////////////////////
// == extra Node function here == //
void atHomeInterrupt() {
  if (!homeDetect) homeDetect = 1;
}

void sendAtHome() {
  if (nToCmd) {
    for (uint8_t i = 0; i < nToCmd; i++) {
      uint8_t n = (flashPin == greenPin ? OFF : ON);
      sendTo(atHomeCmd[i][n]);
    }
  }
}

void atHomeLedOff() {
  digitalWrite(greenPin, anodeOnVCC);
  digitalWrite(redPin, anodeOnVCC);
}

void atHomeFlash() {
  if (homeDetect == 1) {
    homeDetect = 2000;
    detectMillis = millis();
    flashPin = (flashPin == greenPin ? redPin : greenPin);
    atHomeLedOff();
    sendAtHome();
  }
  flashIsOn = !flashIsOn;
  digitalWrite(flashPin, (anodeOnVCC ? !flashIsOn : flashIsOn)); 
  atHomeFlashTime = (flashIsOn ? flashOn : flashOff);
  flashMillis = millis();
}

#endif
