////////////////////////////////////////////
// specify the funcion Items of this Node //
////////////////////////////////////////////

// ==========================================================//
//  the functions of this Node                               //
//  garage/carport: lamp, red flash and park line laser      // 
// ==========================================================//
//  edit the specific function Items code in:                //
//  -----------------------------------------                //
//  "myST_Start.h"   is called from setup()                  //
//  "myST_Cmd.h"     is called on a received command         //
//  "myST_Actiont.h" is, once per minute, called from loop() //
//  if needed: extra onStart/Command/Action functions        //
//    bellow in this header file                             //
// ==========================================================//

#ifndef _MY_ST_NODE_H_
#define _MY_ST_NODE_H_

#include "myST_Gen.h"

const uint8_t OTA_Pin = 15,  // OTA upload: touch on PIN<n> or use the WS command "|OTA|" ...
              threshold = 40,  // OTA and home touch threshold
              lampPin = 23,
              flashPin = 22,
              laserPin = 21,
              flashOn = 50; 
const uint16_t flashOff = 1700;
boolean flashIsOn = false,
        anodeOnVCC = true;       // false: pin-HIGH LED ON, true: pin-HIGH LED OFF  
time_t flashTime,
       flashMillis = millis();

// a @ sign mean do NOT change this name
// =====================================
enum {  // this Node Items
  nLamp,   // Item 1
  nFlash,  // Item 2
  nLaser,  // Item 3
  nItems,  //@ the number of items in use on this Node
};
enum {  // basic field names
  nName,    //@
  nMode,    //@
  nStatus,  //@
  nPause    //@
};
enum {  // Item 1 special field names
  nRise1 = nPause + 1,  // +/- minutes offset on sunrise
  nSet1,                // +/- minutes offset on sunset
  nOnhh1,               // morning ON time before sunrise, format hh (after new day time 04:00)
  nOnmm1,               // morning ON time before sunrise, format mm (after new day time 04:00)
  nOffhh1,              // evening/night OFF time after sunset, format hh (evening or night to 03:59)
  nOffmm1,              // evening/night OFF time after sunset, format mm (evening or night to 03:59)
  nFields1  //@
};
enum {  // Item 2 special field names
  nOnhh2 = nPause + 1,  // +/- minutes offset on sunrise
  nOnmm2,               // morning ON time before sunrise, format mm (after new day time 04:00)
  nOffhh2,              // evening/night OFF time after sunset, format hh (evening or night to 03:59)
  nOffmm2,              // evening/night OFF time after sunset, format mm (evening or night to 03:59)
  nFields2  //@
};
enum {  // Item 3 special field names
  nRise3 = nPause + 1,  // +/- minutes offset on sunrise
  nSet3,                // +/- minutes offset on sunset
  nFields3  //@
};
const uint8_t nFields = (nFields1 > nFields2 ? nFields1 : (nFields2  > nFields3 ? nFields2 : nFields3)),  // the maximum number of used Node Items fields
              itemFields[nItems] = {nFields1, nFields2, nFields3},  // the number of fields Items[0], Item[1}, ... , Item[n]
              nMaxChars = (10 + 1);  // max number of chars in the <...Fields[]> and <...Values[]>
const uint16_t maxItemLen = nFields * nMaxChars + nFields + 1,
               eepromAddr[nItems] = {10, (10 + maxItemLen), (10 + 2 * maxItemLen)};  // Item EEPROM start addresses (start after: OTA[0], address[1-6], and useDST[7])
char fldIn[nFields][nMaxChars],  // parse received message to fieldnames an fieldValues 
     valIn[nFields][nMaxChars];  //   ...
const char itemNames[nItems][nFields][nMaxChars] =  {{"name",  "mode", "status", "pause", "rise",  "set",   "on-hh",  "on-mm", "off-hh", "off-mm"},
                                                     {"name",  "mode", "status", "pause", "on-hh", "on-mm", "off-hh", "off-mm"},
                                                     {"name",  "mode", "status", "pause", "rise",  "set"}},
           itemValOrg[nItems][nFields][nMaxChars] = {{"lamp",  "3",    "0",       "0",    "10",    "10",    "07",     "30",    "24",     "00"},
                                                     {"flash", "1",    "0",       "0",    "04",    "00",    "03",     "59"},
                                                     {"laser", "1",    "0",       "0",    "0",     "0"}};
                                                     // mode 0: manual
char itemValues[nItems][nFields][nMaxChars];         // lamp mode: 1: sunrise, 2:sunset, 3: sunrise and sunset
char cmdList[nItems][10] = {{"nothing|."},{"nothing|."},{"nothing|."}};  // set additional command list or "nothing", put code in myST_Cmd.h->onCommand() 

/** general used functions **/
void fromSetup();  // wath to do from setup()?
void fromLoop_1();   // wath to do from loop() before onAction()?
void fromLoop_2();   // wath to do from loop() after onAction()?
uint8_t parseFldVal(char * mesage);  // used by onCommand() to parse the input char string

///////////////////////////////////////
/** spesial functions for this Thing **/
///////////////////////////////////////

/** to do from setup() here **/
void fromSetup() {
  /** init this Node Item(s) pins and ... here **/
  pinMode(lampPin, OUTPUT);
  pinMode(flashPin, OUTPUT);
  pinMode(laserPin, OUTPUT);
}
 
/** to do from loop() here **/
void fromLoop_1() {
  // before onAction()
}
 
/** to do from loop() here **/
void fromLoop_2() {
  if (getMillis(flashMillis) >= flashTime) {
    flashIsOn = !flashIsOn;
    digitalWrite(flashPin, (anodeOnVCC ? !flashIsOn : flashIsOn)); 
    flashTime = (flashIsOn ? flashOn : flashOff);
    flashMillis = millis();
  }
}

/** parse the received message to fieldnames an fieldValues **/ 
uint8_t parseFldVal(char * mesage) {
  uint8_t n = 0;
  char * v;
  /** get the number of fields **/
  v = strchr(mesage, '|');
  while (v != NULL) {
    n++;
    v = strchr(v + 1, '|');
  }
  if (n > 3 && (n % 2 != 0)) { // <= 3:no data, 'n' not neg. wrong format
    n = (n - 1) / 2;;
    uint8_t i = 0;
    /** get the fields **/
    v = strtok (mesage, "|");
    while (v != NULL) {
      strcpy(fldIn[i], v);
      v = strtok (NULL, "|");
      strcpy(valIn[i++], v);
      v = strtok (NULL, "|");
    }
  }
  else n = 0;
  return n;
}

///////////////////////////////////////
// == extra Node function here == //

#endif
