////////////////////////////////////////////
// specify the funcion Items of this Node //
////////////////////////////////////////////

// ==========================================================//
//  the functions of this Node                               //
//  garten lamp                                              // 
// ==========================================================//
//  edit the specific function Items code in:                //
//  -----------------------------------------                //
//  "myST_Start.h"   is called from setup()                  //
//  "myST_Cmd.h"     is called on a received command         //
//  "myST_Actiont.h" is, once per minute, called from loop() //
//  if needed: extra onStart/Command/Action functions        //
//    bellow in this header file                             //
// ==========================================================//

#ifndef _MY_ST_NODE_H_
#define _MY_ST_NODE_H_

#include "myST_Gen.h"

const uint8_t OTA_Pin = 15,  // OTA upload: touch on PIN<n> or use the WS command "|OTA|" ...
              threshold = 40,  // OTA and home touch threshold
              lampPin = 21; 

// a @ sign mean do NOT change this name
// =====================================
enum {  // this Node Items
  nLamp,  // Item 1
  nItems,  //@ the number of items in use on this Node
};
enum {  // basic field names
  nName,    //@
  nMode,    //@
  nStatus,  //@
  nPause    //@
};
enum {  // Item 1 special field names
  nRise1 = nPause + 1,  // +/- minutes offset on sunrise
  nSet1,                // +/- minutes offset on sunset
  nOnhh1,               // morning ON time before sunrise, format hh (after new day time 04:00)
  nOnmm1,               // morning ON time before sunrise, format mm (after new day time 04:00)
  nOffhh1,              // evening/night OFF time after sunset, format hh (evening or night to 03:59)
  nOffmm1,              // evening/night OFF time after sunset, format mm (evening or night to 03:59)
  nFields1  //@
};
const uint8_t nFields = nFields1,  // the maximum number of used Node Items fields
              itemFields[nItems] = {nFields1},  // the number of fields Items[0], Item[1}, ... , Item[n]
              nMaxChars = (10 + 1);  // max number of chars in the <...Fields[]> and <...Values[]>
const uint16_t maxItemLen = nFields * nMaxChars + nFields + 1,
               eepromAddr[nItems] = {10};  // Item EEPROM start addresses (start after: OTA[0], address[1-6], and useDST[7])
char fldIn[nFields][nMaxChars],  // parse received message to fieldnames an fieldValues 
     valIn[nFields][nMaxChars];  //   ...
const char itemNames[nItems][nFields][nMaxChars] =  {"name", "mode", "status", "pause", "rise", "set", "on-hh", "on-mm", "off-hh", "off-mm"},
           itemValOrg[nItems][nFields][nMaxChars] = {"lamp", "3",    "0",       "0",    "10",   "10",  "07",    "00",    "23",     "30"};
char itemValues[nItems][nFields][nMaxChars];               // mode 0:  1: sunrise, 2:sunset, 3: sunrise and sunset
char cmdList[nItems][10] = {"nothing|."};  // set additional command list or "nothing", put code in myST_Cmd.h->onCommand() 

/** general used functions **/
void fromSetup();  // wath to do from setup()?
void fromLoop_1();   // wath to do from loop() before onAction()?
void fromLoop_2();   // wath to do from loop() after onAction()?
uint8_t parseFldVal(char * mesage);  // used by onCommand() to parse the input char string

///////////////////////////////////////
/** spesial functions for this Thing **/

///////////////////////////////////////

/** to do from setup() here **/
void fromSetup() {
  /** init this Node Item(s) pins and ... here **/
  pinMode(lampPin, OUTPUT);
}
 
/** to do from loop() here **/
void fromLoop_1() {
  // before onAction()
}
 
/** to do from loop() here **/
void fromLoop_2() {
}

/** parse the received message to fieldnames an fieldValues **/ 
uint8_t parseFldVal(char * mesage) {
  uint8_t n = 0;
  char * v;
  /** get the number of fields **/
  v = strchr(mesage, '|');
  while (v != NULL) {
    n++;
    v = strchr(v + 1, '|');
  }
  if (n > 3 && (n % 2 != 0)) { // <= 3:no data, 'n' not neg. wrong format
    n = (n - 1) / 2;;
    uint8_t i = 0;
    /** get the fields **/
    v = strtok (mesage, "|");
    while (v != NULL) {
      strcpy(fldIn[i], v);
      v = strtok (NULL, "|");
      strcpy(valIn[i++], v);
      v = strtok (NULL, "|");
    }
  }
  else n = 0;
  return n;
}

///////////////////////////////////////
// == extra Node function here == //

#endif
